-- Masp_buscar_personasySQL Administrator dump 1.4
--
-- ------------------------------------------------------
-- Server version	5.0.18-nt


/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES utf8 */;

/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;


--
-- Create schema agenda
--

CREATE DATABASE /*!32312 IF NOT EXISTS*/ Agenda;
USE Agenda;

--
-- Table structure for table `agenda`.`personas`
--

DROP TABLE IF EXISTS `Personas`;
CREATE TABLE `Personas` (
  `IDPersona` int(11) NOT NULL,
  `Apellidos` varchar(100) NOT NULL,
  `Nombres` varchar(100) NOT NULL,
  `FechaNacimiento` date default NULL,
  `Domicilio` varchar(100) default NULL,
  `Email` varchar(100) default NULL,
  PRIMARY KEY  (`IDPersona`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

--
-- Dumping data for table `agenda`.`personas`
--

/*!40000 ALTER TABLE `Personas` DISABLE KEYS */;
INSERT INTO `Personas` (`IDPersona`,`Apellidos`,`Nombres`,`FechaNacimiento`,`Domicilio`,`Email`) VALUES 
 (1,'Paolini','Maximiliano','1973-03-02','sdssffs','3e3534545'),
 (2,'Goñi','Martín','1980-01-30','Av. Sarmiento 715','mgoni@herrera.unt.edu.ar');
/*!40000 ALTER TABLE `Personas` ENABLE KEYS */;


--
-- Table structure for table `agenda`.`telefonos`
--

DROP TABLE IF EXISTS `Telefonos`;
CREATE TABLE `Telefonos` (
  `IDPersona` int(11) NOT NULL,
  `IDTelefono` int(11) NOT NULL,
  `Tipo` varchar(50) NOT NULL,
  `Numero` varchar(20) NOT NULL,
  PRIMARY KEY  (`IDPersona`,`IDTelefono`),
  KEY `Ref11` (`IDPersona`),
  CONSTRAINT `RefPersonas` FOREIGN KEY (`IDPersona`) REFERENCES `Personas` (`IDPersona`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

--
-- Dumping data for table `agenda`.`telefonos`
--

/*!40000 ALTER TABLE `Telefonos` DISABLE KEYS */;
INSERT INTO `Telefonos` (`IDPersona`,`IDTelefono`,`Tipo`,`Numero`) VALUES 
 (1,1,'Celular','156400489'),
 (2,1,'Particular','4221949');
/*!40000 ALTER TABLE `Telefonos` ENABLE KEYS */;


--
-- Procedure `agenda`.`asp_alta_persona`
--

DROP PROCEDURE IF EXISTS `asp_alta_persona`;
DELIMITER //
CREATE PROCEDURE `asp_alta_persona`(ape varchar(100), nom varchar (100), fec date, dir varchar(100), mail varchar (100), OUT mensaje varchar(100))
BEGIN
	DECLARE idp integer;

	IF ape = '' OR nom = '' THEN
		SET mensaje = 'El apellido y/o nombre están vacíos';
	ELSE
		START TRANSACTION;
			SET idp = (SELECT COALESCE(MAX(IDPersona), 0) + 1 FROM Personas);
			INSERT INTO Personas VALUES (idp, ape, nom, fec, dir, mail);
			SET Mensaje = 'Persona creada';
		COMMIT;
	END IF;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_alta_telefono`
--

DROP PROCEDURE IF EXISTS `asp_alta_telefono`;
DELIMITER //
CREATE PROCEDURE `asp_alta_telefono`(idp integer, tipo varchar(50), num varchar(20), OUT mensaje varchar(100))
BEGIN
	DECLARE idt integer;
	
	IF tipo = '' OR num = '' THEN
		SET mensaje = 'El tipo y/o número están vacíos';
	ELSE
		START TRANSACTION;
			SET idt = (SELECT COALESCE(MAX(IDTelefono), 0) + 1 FROM Telefonos WHERE IDPersona = idp);
			INSERT INTO Telefonos VALUES (idp, idt, tipo, num);
			SET mensaje = 'Teléfono creado';
		COMMIT;
	END IF;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_borrar_persona`
--

DROP PROCEDURE IF EXISTS `asp_borrar_persona`;
DELIMITER //
CREATE PROCEDURE `asp_borrar_persona`(idp integer, OUT mensaje varchar(100))
BEGIN
	IF EXISTS (SELECT IDPersona FROM Telefonos WHERE IDPersona = idp) THEN
		SET mensaje = 'No se puede borrar la persona porque tiene teléfonos asociados';
	ELSE
		DELETE FROM Personas WHERE IDPersona = idp;
		SET mensaje = 'Persona borrada';
	END IF;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_borra_telefono`
--

DROP PROCEDURE IF EXISTS `asp_borrar_telefono`;
DELIMITER //
CREATE PROCEDURE `asp_borrar_telefono`(idp int, idt int, OUT mensaje varchar(100))
BEGIN
	IF EXISTS (SELECT * FROM Telefonos WHERE IDPersona = idp AND IDTelefono = idt) THEN
		DELETE FROM Telefonos WHERE IDPersona = idp AND IDTelefono = idt;
		SET mensaje = 'Teléfono borrado';
	ELSE
		SET mensaje = 'No existe el teléfono y/o persona especificados';
	END IF;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_buscar_personas`
-- Busca las personas que comiencen con los caracteres especificados
-- Si la cadena de búsqueda está vacía, busca todas las personas
--

DROP PROCEDURE IF EXISTS `asp_buscar_personas`;
DELIMITER //
CREATE PROCEDURE `asp_buscar_personas`(cad varchar (100))
BEGIN
	SELECT *
	FROM Personas
	WHERE Apellidos LIKE CONCAT(cad ,'%')
	ORDER BY Apellidos;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_calendario`
--

DROP PROCEDURE IF EXISTS `asp_calendario`;
DELIMITER //
CREATE PROCEDURE `asp_calendario`()
BEGIN
	SELECT	DAY(FechaNacimiento) 'Dia', MONTH(FechaNacimiento) 'Mes', CONCAT(Nombres,' ',Apellidos) 'Nombre'
	FROM Personas
	ORDER BY 1, 2, 3;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_dame_persona`
--

DROP PROCEDURE IF EXISTS `asp_dame_persona`;
DELIMITER //
CREATE PROCEDURE `asp_dame_persona`(idp integer)
BEGIN
	SELECT * FROM Personas
	WHERE IDPersona = idp;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_dame_telefono`
--

DROP PROCEDURE IF EXISTS `asp_dame_telefono`;
DELIMITER //
CREATE PROCEDURE `asp_dame_telefono`(idp integer,idt integer)
BEGIN
	SELECT * FROM Telefonos
	WHERE IDPersona = idp AND IDTelefono = idt;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_dame_telefonos`
--

DROP PROCEDURE IF EXISTS `asp_dame_telefonos`;
DELIMITER //
CREATE PROCEDURE `asp_dame_telefonos`(idp integer)
BEGIN
	SELECT IDPersona, IDTelefono, Tipo, Numero
	FROM Telefonos
	WHERE IDPersona = idp;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_hoy_cumple`
--

DROP PROCEDURE IF EXISTS `asp_hoy_cumple`;
DELIMITER //
CREATE PROCEDURE `asp_hoy_cumple`()
BEGIN
	SELECT	CONCAT(apellidos, ', ', nombres) 'Nombre'
	FROM Personas
	WHERE MONTH(FechaNacimiento) = MONTH(CURDATE()) AND DAY(FechaNacimiento) = DAY(CURDATE())
	ORDER BY 1;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_listado`
--

DROP PROCEDURE IF EXISTS `asp_listado`;
DELIMITER //
CREATE PROCEDURE `asp_listado`()
BEGIN
	SELECT P.IDPersona, Apellidos, Nombres, Domicilio, Email, Tipo, Numero
	FROM Personas P INNER JOIN Telefonos T 
    ON (P.IDPersona = T.IDPersona)
	ORDER BY 2, 3;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_modifica_persona`
--

DROP PROCEDURE IF EXISTS `asp_modificar_persona`;
DELIMITER //
CREATE PROCEDURE `asp_modificar_persona`(idp integer, ape varchar(100), nom varchar (100), fec datetime, dir varchar(100), mail varchar (100), OUT mensaje varchar(100))
BEGIN
	IF ape = '' OR nom = '' THEN
		SET mensaje = 'El apellido y/o nombre están vacíos';
	ELSE
		UPDATE Personas
		SET Apellidos = ape, Nombres = nom, FechaNacimiento = fec, Domicilio = dir, Email = mail
		WHERE IDPersona = idp;
		SET Mensaje = 'Persona modificada';
	END IF;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_modifica_telefono`
--

DROP PROCEDURE IF EXISTS `asp_modificar_telefono`;
DELIMITER //
CREATE PROCEDURE `asp_modificar_telefono`(idp int, idt int, tipo varchar(50), num varchar (20), OUT mensaje varchar(100))
BEGIN
	IF tipo = '' OR num = '' THEN
		SET mensaje = 'El tipo y/o número están vacíos';
	ELSE
		UPDATE Telefonos 
        SET Tipo = tipo, Numero = num
		WHERE IDPersona = idp AND IDTelefono = idt;
		SET Mensaje = 'Teléfono modificado';
	END IF;
END //
DELIMITER ;

--
-- Procedure `agenda`.`asp_total_personas`
--

DROP PROCEDURE IF EXISTS `asp_total_personas`;
DELIMITER //
CREATE PROCEDURE `asp_total_personas`(OUT cantidad integer)
BEGIN
	SET cantidad = (SELECT COALESCE(COUNT(IDPersona), 0) FROM Personas);
END //
DELIMITER ;

--
-- Usuario y permisos
--
-- GRANT ALL PRIVILEGES ON `Agenda`.* to 'Estudiante'@'localhost' identified by 'estudiante';
-- Crea el usuario Estudiante (si no existe), con clave estudiante, 

CREATE USER 'Estudiante'@'localhost' IDENTIFIED BY 'estudiante';
GRANT EXECUTE ON `Agenda`.* TO 'Estudiante'@'localhost';



/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
