USE [master]
GO

/****** BD ******/
CREATE DATABASE [Agenda] ON  PRIMARY 
 (NAME = N'Agenda', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10_50.MSSQLSERVER\MSSQL\DATA\Agenda.mdf' , SIZE = 3072KB , MAXSIZE = UNLIMITED, FILEGROWTH = 1024KB )
LOG ON 
 (NAME = N'Agenda_log', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10_50.MSSQLSERVER\MSSQL\DATA\Agenda_log.ldf' , SIZE = 1024KB , MAXSIZE = 2048GB , FILEGROWTH = 10%)
GO


/****** Tablas ******/
USE [Agenda]
GO

CREATE TABLE [dbo].[Personas] (
	[IdPersona] [int] NOT NULL,
	[Apellidos] [varchar] (100),
	[Nombres] [varchar] (100),
	[FechaNacimiento] [datetime] NULL,
	[Domicilio] [varchar] (100) NULL,
	[Email] [varchar] (100) NULL 
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[Telefonos] (
	[IdPersona] [int] NOT NULL ,
	[IdTelefono] [int] NOT NULL ,
	[Tipo] [varchar] (50) NOT NULL,
	[Numero] [varchar] (20) NOT NULL 
) ON [PRIMARY]
GO

/****** Restricciones ******/

ALTER TABLE [dbo].[Personas] WITH NOCHECK 
ADD CONSTRAINT [PK1] PRIMARY KEY  NONCLUSTERED ([IdPersona])  ON [PRIMARY] 
GO

ALTER TABLE [dbo].[Telefonos] WITH NOCHECK 
ADD CONSTRAINT [PK2] PRIMARY KEY  NONCLUSTERED ([IdPersona], [IdTelefono]) ON [PRIMARY] 
GO

ALTER TABLE [dbo].[Telefonos] 
ADD CONSTRAINT [RefPersonas1] FOREIGN KEY ([IdPersona]) REFERENCES [Personas] ([IdPersona])
GO

/****** ndices ******/
CREATE  INDEX [ixIdPersona] ON [Telefonos]([IdPersona]) ON [PRIMARY]
GO

/****** Datos iniciales ******/
set dateformat 'dmy'
INSERT INTO Personas VALUES (1, 'Paolini', 'Maximiliano', '02/03/1973', 'sdssffs', '3e3534545')
INSERT INTO Personas VALUES (2, 'Goi', 'Martn', '30/01/1980', 'Av. Sarmiento 715', 'mgoni@herrera.unt.edu.ar')

INSERT INTO Telefonos VALUES (1, 1, 'Celular', '156400489')
INSERT INTO Telefonos VALUES (2, 1, 'Particular', '4221949')

/****** Procedimientos ******/
SET QUOTED_IDENTIFIER OFF 
GO

CREATE PROCEDURE asp_alta_persona
  @ape varchar(100),
  @nom varchar (100), 
  @fec smalldatetime,
  @dir varchar(100),
  @mail varchar (100),
  @mensaje varchar(100) output
AS
  DECLARE @idp int
  IF @ape = '' OR @nom = ''
  BEGIN
    SET @mensaje = 'El apellido y/o nombre estn vacos'
  END
  BEGIN TRY
	BEGIN TRAN
		SELECT @idp = 1 + COALESCE(MAX(IdPersona), 0) FROM Personas
		INSERT INTO Personas VALUES (@idp, @ape, @nom, @fec, @dir, @mail)
	COMMIT TRAN
	SET @mensaje = 'Persona creada'   
  END TRY
  BEGIN CATCH
	ROLLBACK TRAN
	SET @mensaje = 'Error al guardar la persona'     
  END CATCH
GO  

CREATE PROCEDURE asp_alta_telefono
  @idp int,
  @tipo varchar(50),
  @num varchar (20),
  @mensaje varchar(100) output
AS
  DECLARE @idt int
  IF @tipo = '' OR @num = ''
  BEGIN
	SET @mensaje = 'El tipo y/o nmero estn vacos'
  END
  BEGIN TRY
	BEGIN TRAN
		SELECT @idt = 1 + COALESCE(MAX(IdTelefono), 0) FROM Telefonos WHERE IdPersona = @idp
		INSERT INTO Telefonos VALUES (@idp, @idt, @tipo, @num)  
	COMMIT TRAN
	SET @mensaje = 'Telfono creado'
  END TRY
  BEGIN CATCH
	ROLLBACK TRAN
    SET @mensaje = 'Error al guardar el telfono'  
  END CATCH
GO  

CREATE PROCEDURE asp_borrar_persona
  @idp int,
  @mensaje varchar(100) output
AS
  IF EXISTS (SELECT IdPersona FROM Telefonos WHERE IdPersona = @idp)
  BEGIN
    SET @mensaje = 'No se puede borrar la persona porque tiene telfonos asociados'
  END
  ELSE
  BEGIN
	DELETE FROM Personas WHERE IdPersona = @idp
	SET @mensaje = 'Persona borrada'
  END
GO

CREATE PROCEDURE asp_borrar_telefono
  @idp int,
  @idt int,
  @mensaje varchar(100) output
AS
  IF EXISTS (SELECT * FROM Telefonos WHERE IdPersona = @idp AND IdTelefono = @idt)
  BEGIN
	DELETE FROM Telefonos WHERE IdPersona = @idp AND IdTelefono = @idt
	set @mensaje = 'Telfono borrado'
  END
  ELSE
  BEGIN
	set @mensaje = 'No existe el telfono y/o persona especificados'
  END
GO

CREATE PROCEDURE asp_buscar_personas
  @cad varchar (100)
AS
  SELECT *
  FROM Personas
  WHERE	Apellidos LIKE @cad + '%'
  ORDER BY Apellidos
GO

CREATE PROCEDURE asp_calendario
AS
  SELECT DAY(FechaNacimiento) AS 'Dia', MONTH(FechaNacimiento) AS 'Mes', Nombres + ' ' + Apellidos AS 'Nombre'
  FROM Personas
  ORDER BY 1,2,3
GO

CREATE PROCEDURE asp_dame_persona
  @idp int
AS
  SELECT *
  FROM Personas
  WHERE	IdPersona = @idp
GO

CREATE PROCEDURE asp_dame_telefono
  @idp int,
  @idt int
AS
  SELECT *
  FROM Telefonos
  WHERE	IdPersona = @idp AND IdTelefono = @idt
GO

CREATE PROCEDURE asp_dame_telefonos
  @idp int
AS
  SELECT idPersona, IdTelefono, Tipo, Numero
  FROM Telefonos
  WHERE	IdPersona = @idp
GO

CREATE PROCEDURE asp_hoy_cumple
AS
  SELECT apellidos + ', ' + nombres AS 'Nombre'
  FROM Personas
  WHERE	MONTH(FechaNacimiento) = MONTH(GETDATE()) AND DAY(FechaNacimiento) = DAY(GETDATE())
  ORDER BY	1
GO

CREATE PROCEDURE asp_listado
AS
  SELECT P.IdPersona, Apellidos, Nombres, Domicilio, Email, Tipo, Numero
  FROM Personas P JOIN Telefonos T 
  ON P.IdPersona = T.IdPersona
  ORDER BY 2, 3
GO

CREATE PROCEDURE asp_modificar_persona
  @idp int,
  @ape varchar(100),
  @nom varchar (100),
  @fec smalldatetime,
  @dir varchar(100),
  @mail varchar (100),
  @mensaje varchar(100) output
AS
  IF @ape = '' OR @nom = ''
  BEGIN
    SET @mensaje = 'El apellido y/o nombre estn vacos'
  END
  BEGIN TRY
    UPDATE Personas
    SET Apellidos = @ape, Nombres = @nom, FechaNacimiento = @fec, Domicilio = @dir, Email = @mail
    WHERE IdPersona = @idp
    SET @mensaje = 'Persona modificada'
  END TRY
  BEGIN CATCH
    SET @mensaje = 'Error al guardar la persona'
  END CATCH
GO

CREATE PROCEDURE asp_modificar_telefono
  @idp int,
  @idt int,
  @tipo varchar(50),
  @num varchar (20),
  @mensaje varchar(100) output
AS
  IF  @tipo = '' OR @num = ''
  BEGIN
	SET @mensaje = 'El tipo y/o nmero estn vacos'
  END
  BEGIN TRY
	UPDATE Telefonos SET Tipo = @tipo, Numero = @num
	WHERE IdPersona = @idp AND IdTelefono = @idt
    SET @mensaje = 'Telfono modificado'	
  END TRY
  BEGIN CATCH
    SET @mensaje = 'Error al guardar el telfono'
  END CATCH
GO

CREATE PROCEDURE asp_total_personas
  @cantidad int output
AS
  set @cantidad = (SELECT COALESCE(COUNT(IdPersona), 0) FROM Personas)
GO

/****** Login ******/
if not exists 
  (select name
   from master.sys.server_principals
   where name = 'Estudiante')
begin
  create login Estudiante with password = 'estudiante'
end

/****** Usuario y permisos ******/
USE [Agenda]
GO

if not exists 
  (select name
   from sys.database_principals
   where name = 'Estudiante')
begin
  create user Estudiante for login Estudiante
end

exec sp_addrolemember 'db_accessadmin', 'Estudiante'

grant execute on dbo.asp_alta_persona to Estudiante
grant execute on dbo.asp_alta_telefono to Estudiante
grant execute on dbo.asp_borrar_persona to Estudiante
grant execute on dbo.asp_borrar_telefono to Estudiante
grant execute on dbo.asp_buscar_personas to Estudiante
grant execute on dbo.asp_calendario to Estudiante
grant execute on dbo.asp_dame_persona to Estudiante
grant execute on dbo.asp_dame_telefono to Estudiante
grant execute on dbo.asp_dame_telefonos to Estudiante
grant execute on dbo.asp_hoy_cumple to Estudiante
grant execute on dbo.asp_listado to Estudiante
grant execute on dbo.asp_modificar_persona to Estudiante
grant execute on dbo.asp_modificar_telefono to Estudiante
grant execute on dbo.asp_total_personas to Estudiante