/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package controladores;

import interfaces.IControladorTelefonos;
import interfaces.IControladorPersonas;
import modelos.Persona;
import modelos.Telefono;
import reportes.PreImpresion;
import vistas.VentanaPersonas;
import com.toedter.calendar.JDateChooser;
import interfaces.IControladorLogin;
import interfaces.IGestorPersonas;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.sql.Connection;
import java.sql.SQLException;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import modelos.Conexion;
import modelos.GestorPersonas;
import modelos.ModeloTablaPersonas;
import modelos.ModeloTablaTelefonos;
import vistas.VentanaCalendario;

public class ControladorPersonas implements IControladorPersonas {
    private VentanaPersonas ventana;   
    private static ControladorPersonas instancia;
    
    /**
     * Constructor
    */    
    private ControladorPersonas() {
        this.ventana = new VentanaPersonas(this);
        
        this.ventana.setIconImage(new ImageIcon(VentanaPersonas.class.getResource("agenda.gif")).getImage());
        this.ventana.setLocationRelativeTo(null);
        this.ventana.verCampoBusca().requestFocus();        
        
        this.mostrarPersonas(""); //Muestra la tabla con todas las personas creadas
        JTable tablaPersonas = this.ventana.verTablaPersonas();
        if (tablaPersonas.getRowCount() > 0) { //si la tabla tiene filas para mostrar
            tablaPersonas.setRowSelectionInterval(0, 0); //selecciona la primer persona de la tabla
            this.tablaPersonasClic(null);  //Muestra en los campos de texto la persona seleccionada en la tabla            
        }
        this.hoyCumple(); //muestra los que cumplen año hoy
        
        this.ventana.setVisible(true);        
    }
    
    /**
     * Método que permite crear una única instancia de ControladorPersonas
     * @return ControladorPersonas  - objeto ControladorPersonas
    */    
    public static ControladorPersonas crear() {
        if (instancia == null)
            instancia = new ControladorPersonas();
        return instancia;
    }

    /**
     * Acción a ejecutar cuando se hace clic en la tabla Personas
     * Si se selecciona una persona, se muestran sus datos en los campos de texto y sus teléfonos
     * @param evt evento
    */            
    @Override
    public void tablaPersonasClic(MouseEvent evt) {
        JTable tablaPersonas = this.ventana.verTablaPersonas();
        int filaSeleccionada = tablaPersonas.getSelectedRow();
        if (filaSeleccionada != -1) {            
            Persona unaPersona = ((ModeloTablaPersonas)tablaPersonas.getModel()).obtenerPersona(filaSeleccionada);
            this.ventana.verCampoApellidos().setText(unaPersona.verApellidos());
            this.ventana.verCampoNombres().setText(unaPersona.verNombres());
            if (unaPersona.verFecNac() != null)
                this.ventana.verFechaNacimiento().setDate(java.sql.Date.valueOf(unaPersona.verFecNac()));
            else
                this.ventana.verFechaNacimiento().setDate(null);
            this.ventana.verCampoDireccion().setText(unaPersona.verDireccion());
            this.ventana.verCampoCorreo().setText(unaPersona.verCorreo());
            this.mostrarTelefonos(unaPersona);
        }        
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Nueva Persona
     * Limpia los campos de texto y la tabla de teléfonos para poder ingresar una persona nueva
     * @param evt evento
    */                    
    @Override
    public void btnNuevaPersonaClic(ActionEvent evt) {
        JTextField tApellidos = this.ventana.verCampoApellidos();
        JTextField tNombres = this.ventana.verCampoNombres();
        JDateChooser dFechaNacimiento = this.ventana.verFechaNacimiento();
        JTextField tDireccion = this.ventana.verCampoDireccion();
        JTextField tCorreo = this.ventana.verCampoCorreo();        
        JTextField tBusca = this.ventana.verCampoBusca();
        tApellidos.setText(null);
        tNombres.setText(null);
        dFechaNacimiento.setDate(null);
        tDireccion.setText(null);
        tCorreo.setText(null);
        tBusca.setText(null);
        this.mostrarTelefonos(null);
        tApellidos.requestFocus();
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Guardar
     * Se leen los datos de la persona y se la guarda en la BD
     * Si la fecha de nacimiento es incorrecta, o nula, no se guarda este dato en la BD
     * @see Persona
    */                        
    @Override
    public void btnGuardarClic(ActionEvent evt) {
        JTextField txtApellidos = this.ventana.verCampoApellidos();
        String apellidos = txtApellidos.getText().trim();
        
        JTextField txtNombres = this.ventana.verCampoNombres();
        String nombres = txtNombres.getText().trim();
        
        JDateChooser dFechaNacimiento = this.ventana.verFechaNacimiento();
        Date fecNacimiento = dFechaNacimiento.getDate();
        LocalDate fechaNacimiento;
        if (fecNacimiento == null)
            fechaNacimiento = null;
        else
            fechaNacimiento = Instant.ofEpochMilli(fecNacimiento.getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
        
        JTextField txtDireccion = this.ventana.verCampoDireccion();
        String direccion = (txtDireccion.getText().trim().equals("") ? null : txtDireccion.getText().trim());

        JTextField txtCorreo = this.ventana.verCampoCorreo();        
        String correo = (txtCorreo.getText().trim().equals("") ? null : txtCorreo.getText().trim());
        
        IGestorPersonas gp = GestorPersonas.crear();
        String resultado = gp.altaPersona(apellidos, nombres, fechaNacimiento, direccion, correo);
        if (resultado.equals(IGestorPersonas.MENSAJE_PERSONA_CREADA))
            this.mostrarPersonas(""); //se muestran todas las personas
        else if (resultado.equals(GestorPersonas.MENSAJE_PERSONA_CREADA_ERROR))
            txtApellidos.requestFocus();
        JOptionPane.showMessageDialog(this.ventana, resultado, IControladorLogin.TITULO_APLICACION, JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Borrar Persona
     * Si se selecciona una persona, se intenta borrarla de la BD
     * @see Persona
    */                            
    @Override
    public void btnBorrarPersonaClic(ActionEvent evt) {
        JTable tablaPersonas = this.ventana.verTablaPersonas();
        int filaSeleccionada = tablaPersonas.getSelectedRow();
        if (filaSeleccionada != -1) { //hay seleccionada una persona
            Persona personaSeleccionada = ((ModeloTablaPersonas)tablaPersonas.getModel()).obtenerPersona(filaSeleccionada);
            int opcionSeleccionada = JOptionPane.showConfirmDialog(this.ventana, CONFIRMACION_BORRADO_PERSONA, IControladorLogin.TITULO_APLICACION, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (opcionSeleccionada == JOptionPane.YES_OPTION) {
                IGestorPersonas gp = GestorPersonas.crear();
                String resultado = gp.borrarPersona(personaSeleccionada);
                if (resultado.equals(IGestorPersonas.MENSAJE_PERSONA_BORRADA)) {
                    this.mostrarPersonas("");
                    if (tablaPersonas.getRowCount() > 0) {
                        tablaPersonas.setRowSelectionInterval(0, 0);
                        this.tablaPersonasClic(null);
                    }
                }
                JOptionPane.showMessageDialog(this.ventana, resultado, IControladorLogin.TITULO_APLICACION, JOptionPane.INFORMATION_MESSAGE);                
            }
        }                        
    }
        
    /**
     * Muestra los teléfonos de la persona especificada
     * Si no se especifica una persona, no muestra ningún teléfono
     * También establece qué columnas se muestran
     * @param unaPersona persona a la que se le muestran sus teléfonos
     * @see Persona
     * @see Telefono
    */                                                            
    private void mostrarTelefonos(Persona persona) {
        ModeloTablaTelefonos mtt = new ModeloTablaTelefonos(persona);
        JTable tablaTelefonos = this.ventana.verTablaTelefonos();
        tablaTelefonos.setModel(mtt);
        tablaTelefonos.getColumn("IDPersona").setMinWidth(0);
        tablaTelefonos.getColumn("IDPersona").setMaxWidth(0);
        tablaTelefonos.getColumn("IDPersona").setPreferredWidth(0);            

        tablaTelefonos.getColumn("IDTelefono").setMinWidth(0);
        tablaTelefonos.getColumn("IDTelefono").setMaxWidth(0);
        tablaTelefonos.getColumn("IDTelefono").setPreferredWidth(0);
        if (tablaTelefonos.getRowCount() > 0)
            tablaTelefonos.setRowSelectionInterval(0, 0);
    }    
    
    /**
     * Muestra las personas cuyo apellido coincida con la cadena de búsqueda, o todas las personas si la cadena está vacía
     * Establece las columnas que muestra la tabla de personas 
     * @param cadenaBusqueda parte del apellido con la cual se buscan las personas
     * @see Persona
     * @see Telefono
    */                                                                
    private void mostrarPersonas(String cadenaBusqueda) {
        ModeloTablaPersonas mtp = new ModeloTablaPersonas(cadenaBusqueda);
        JTable tablaPersonas = this.ventana.verTablaPersonas();
        tablaPersonas.setModel(mtp);
        tablaPersonas.getColumn("IDPersona").setMinWidth(0);
        tablaPersonas.getColumn("IDPersona").setMaxWidth(0);
        tablaPersonas.getColumn("IDPersona").setPreferredWidth(0);        
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Nuevo Teléfono
     * Si hay una persona seleccionada, se muestra la ventana para agregarle un nuevo teléfono
     * @param evt evento
    */                                
    @Override
    public void btnNuevoTelClic(ActionEvent evt) {
        JTable tablaPersonas = this.ventana.verTablaPersonas();
        int filaSeleccionada = tablaPersonas.getSelectedRow();        
        if (filaSeleccionada != -1) {
            Persona personaSeleccionada = ((ModeloTablaPersonas)tablaPersonas.getModel()).obtenerPersona(filaSeleccionada);            
            IControladorTelefonos controlador = ControladorTelefonos.crear(this.ventana, personaSeleccionada);
            ControladorTelefonos.anular();
            this.mostrarTelefonos(personaSeleccionada); //actualiza la tabla de teléfonos
        }
        else
            JOptionPane.showMessageDialog(this.ventana, PERSONA_SIN_SELECCIONAR, IControladorLogin.TITULO_APLICACION, JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Modificar Teléfono
     * Si hay seleccionada una persona y un teléfono, se muestra la ventana para modificarle el teléfono
     * @param evt evento
    */                                
    @Override
    public void btnModificarTelClic(ActionEvent evt) {        
        JTable tablaTelefonos = this.ventana.verTablaTelefonos();
        int filaSeleccionadaTel = tablaTelefonos.getSelectedRow(); 
        if (filaSeleccionadaTel != -1) {
            JTable tablaPersonas = this.ventana.verTablaPersonas();
            int filaSeleccionadaPersona = tablaPersonas.getSelectedRow();        
            Persona personaSeleccionada = ((ModeloTablaPersonas)tablaPersonas.getModel()).obtenerPersona(filaSeleccionadaPersona);
            Telefono telefonoSeleccionado = ((ModeloTablaTelefonos)tablaTelefonos.getModel()).obtenerTelefono(filaSeleccionadaTel);
            IControladorTelefonos controlador = ControladorTelefonos.crear(this.ventana, personaSeleccionada, telefonoSeleccionado);
            ControladorTelefonos.anular();
            this.mostrarTelefonos(personaSeleccionada); //actualiza la tabla de teléfonos
        }
        else
            JOptionPane.showMessageDialog(this.ventana, TELEFONO_SIN_SELECCIONAR, IControladorLogin.TITULO_APLICACION, JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Borrar Teléfono
     * Si hay seleccionada una persona y un teléfono, se borra el mismo
     * @param evt evento
    */                                    
    @Override
    public void btnBorrarTelClic(ActionEvent evt) {
        JTable tablaTelefonos = this.ventana.verTablaTelefonos();
        int filaSeleccionadaTel = tablaTelefonos.getSelectedRow(); 
        if (filaSeleccionadaTel != -1) {
            JTable tablaPersonas = this.ventana.verTablaPersonas();
            int filaSeleccionadaPersona = tablaPersonas.getSelectedRow();        
            Persona personaSeleccionada = ((ModeloTablaPersonas)tablaPersonas.getModel()).obtenerPersona(filaSeleccionadaPersona);
            Telefono telefonoSeleccionado = ((ModeloTablaTelefonos)tablaTelefonos.getModel()).obtenerTelefono(filaSeleccionadaTel);
            int opcionSeleccionada = JOptionPane.showConfirmDialog(this.ventana, CONFIRMACION_BORRADO_TELEFONO, IControladorLogin.TITULO_APLICACION, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (opcionSeleccionada == JOptionPane.YES_OPTION) {
                String aux = personaSeleccionada.borrarTelefono(telefonoSeleccionado);
                JOptionPane.showMessageDialog(this.ventana, aux, IControladorLogin.TITULO_APLICACION, JOptionPane.INFORMATION_MESSAGE);
                this.mostrarTelefonos(personaSeleccionada);             
            }
        }
        else
            JOptionPane.showMessageDialog(this.ventana, TELEFONO_SIN_SELECCIONAR, IControladorLogin.TITULO_APLICACION, JOptionPane.ERROR_MESSAGE);        
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Salir
     * Termina la aplicación
     * @param evt evento
    */                                    
    @Override
    public void btnSalirClic(ActionEvent evt) {
        this.ventana.dispose();
        Connection cnn = Conexion.verConnection();
        try {
            if (cnn != null)
                cnn.close();
            System.exit(0);
        }
        catch(SQLException e) {
            String mensajeError = "Error al cerrar la conexión";
            JOptionPane.showMessageDialog(null, mensajeError, "Agenda", JOptionPane.ERROR_MESSAGE);                                    
        }        
        System.exit(0);
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Buscar
     * Si se especifica una cadena de búsqueda, se muestran las personas cuyos apellidos comiencen con esta cadena
     * Si no se especifica una cadena, se muestran todas las personas
     * @param evt evento
    */                                    
    @Override
    public void btnBuscarClic(ActionEvent evt) {
        JTextField txtBusca = this.ventana.verCampoBusca();
        String cadenaBusqueda = txtBusca.getText().trim();
        this.mostrarPersonas(cadenaBusqueda);
        JTable tablaPersonas = this.ventana.verTablaPersonas();
        if (tablaPersonas.getRowCount() > 0) {
            tablaPersonas.setRowSelectionInterval(0, 0);
                tablaPersonasClic(null); 
        }
    }

    /**
     * Acción a ejecutar cuando se presiona una tecla en el campo txtBusca
     * Si se presiona Enter, se muestran las personas cuyos apellidos comiencen con la cadena de búsqueda, o todas si no se especifica una cadena
     * @see Persona
    */                                    
    @Override
    public void txtBuscaPresionarTecla(KeyEvent evt) {
        if (evt.getKeyCode()== evt.VK_ENTER) {
            this.btnBuscarClic(null);
        }        
    }

    /**
     * Implementación del método que se dispara cuando el campo donde se ingresa la cadena de búsqueda obtiene el foco
     * Se selecciona todo el texto
     * @param evt evento
    */                                        
    @Override
    public void txtBuscaGanaFoco(FocusEvent evt) {
        this.ventana.verCampoBusca().selectAll();
    }

    /**
     * Implementación del método que se dispara cuando el campo donde se ingresa el obtiene el foco
     * Se selecciona todo el texto
     * @param evt evento
    */                                            
    @Override
    public void txtCorreoGanaFoco(FocusEvent evt) {
        this.ventana.verCampoCorreo().selectAll();
    }

    /**
     * Implementación del método que se dispara cuando el campo donde se ingresa la dirección obtiene el foco
     * Se selecciona todo el texto
     * @param evt evento
    */                                            
    @Override
    public void txtDireccionGanaFoco(FocusEvent evt) {
        this.ventana.verCampoDireccion().selectAll();
    }

    /**
     * Implementación del método que se dispara cuando el campo donde se ingresa el nombre obtiene el foco
     * Se selecciona todo el texto
     * @param evt evento
    */                                            
    @Override
    public void txtNombresGanaFoco(FocusEvent evt) {
        this.ventana.verCampoNombres().selectAll();
    }

    /**
     * Implementación del método que se dispara cuando el campo donde se ingresa el apellido obtiene el foco
     * Se selecciona todo el texto
     * @param evt evento
    */                                                
    @Override
    public void txtApellidosGanaFoco(FocusEvent evt) {
        this.ventana.verCampoApellidos().selectAll();
    }

    /**
     * Muestra los que cumplen año hoy
     */
    private void hoyCumple() {
        IGestorPersonas gp = GestorPersonas.crear();
        String aux = gp.hoyCumple();
        if (!aux.isEmpty()) {
            JOptionPane.showMessageDialog(this.ventana, aux, IControladorLogin.TITULO_APLICACION, JOptionPane.INFORMATION_MESSAGE);
        }           
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Cumples
     * @param evt evento
    */                                                        
    @Override
    public void btnCumplesClic(ActionEvent evt) {
        VentanaCalendario v = new VentanaCalendario(this.ventana);
        v.setVisible(true);
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Listado
     * @param evt evento
     */                                    
    @Override
    public void btnListadoClic(ActionEvent evt) {
        String archivo = PreImpresion.class.getResource("listado.jasper").getFile();
        archivo = archivo.replace("%20"," ");
        //Si el nombre del archivo tuviera espacios en blanco, los mismos se leen como %20
        //con esta línea se reemplazan los %20 por espacios en blanco
        HashMap<String, Object> parametros = new HashMap<String, Object>();
        //Ejemplo de parámetro: parametros.put("IDPersona", <valor>);
        Connection cnn = Conexion.verConnection();
        PreImpresion p = new PreImpresion(archivo, cnn, parametros);        
        p.mostrar();        
    }
    
        

    

    
}
