/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package controladores;

import interfaces.IControladorTelefonos;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import modelos.Persona;
import modelos.Telefono;
import vistas.VentanaPersonas;
import vistas.VentanaTelefono;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import modelos.ModeloComboTipoTelefono;

public class ControladorTelefonos implements IControladorTelefonos {
    private VentanaTelefono ventana;  
    private Persona persona;
    private Telefono telefono;
    private static ControladorTelefonos instancia;
    
    /**
     * Constructor
     * @param ventanaPersonas ventana padre (VentanaPersonas en este caso)
     * @param persona persona a la que se crea o modifica un teléfono
     * @param telefono teléfono a modificar
     */
    private ControladorTelefonos(VentanaPersonas ventanaPersonas, Persona persona, Telefono telefono) {      
        this.ventana = new VentanaTelefono(ventanaPersonas, this);
        this.ventana.setLocationRelativeTo(null);
        this.persona = persona;
        this.telefono = telefono;
        JTextField txtNumero = this.ventana.verCampoNumero();
        JComboBox comoTipo = this.ventana.verComboTipo();
        ModeloComboTipoTelefono mctt = new ModeloComboTipoTelefono();
        comoTipo.setModel(mctt);
        
        if (this.telefono == null) { //nuevo teléfono
            this.ventana.setTitle(NUEVO_TELEFONO);
        } 
        else { //modificar teléfono
            this.ventana.setTitle(MODIFICAR_TELEFONO);
            mctt.seleccionarTipoTelefono(telefono.obtenerTipo());
            txtNumero.setText(this.telefono.verNumero());
        }
        txtNumero.requestFocus();
        this.ventana.setVisible(true);        
    }

    /**
     * Método que permite crear una única instancia de ControladorPersonas (para crear un teléfono)
     * @param ventanaPersonas VentanaPersonas
     * @param persona persona a la que se crea o modifica un teléfono
     * @return ControladorPersonas  - objeto ControladorPersonas
    */        
    public static ControladorTelefonos crear(VentanaPersonas ventanaPersonas, Persona persona) {
        return crear(ventanaPersonas, persona, null);
    }
    
    /**
     * Método que permite crear una única instancia de ControladorPersonas (para modificar un teléfono)
     * @param ventanaPersonas VentanaPersonas
     * @param persona persona a la que se crea o modifica un teléfono
     * @param telefono teléfono a modificar
     * @return ControladorPersonas  - objeto ControladorPersonas
    */        
    public static ControladorTelefonos crear(VentanaPersonas ventanaPersonas, Persona persona, Telefono telefono) {
        if (instancia == null)
            instancia = new ControladorTelefonos(ventanaPersonas, persona, telefono);
        return instancia;
    }

    /**
     * Este método permite volver a instanciar esta clase
     * Sin este método, la primera vez que instancia la clase (para agregar/modificar) un teléfono
     * implica que la variable instancia apunta a un ControladorTelefonos
     * Luego, al querer volver a crear/modificar un teléfono, instancia sigue apuntando a un ControladorTelefonos
     * y por lo tanto no muestra la ventana.
     */
    public static void anular() {
        instancia = null;
    }    

    /**
     * Acción a ejecutar cuando se selecciona el botón Cancelar
     * @param evt evento
     */                                    
    @Override
    public void btnCancelarClic(ActionEvent evt) {
        this.ventana.setVisible(false);
    }

    /**
     * Acción a ejecutar cuando el campo txtNumero gana el foco
     * @param evt evento
     */                                    
    @Override
    public void txtNumeroGanaFoco(FocusEvent evt) {
        JTextField txtNumero = this.ventana.verCampoNumero();
        txtNumero.selectAll();
    }

    /**
     * Acción a ejecutar cuando se selecciona el botón Aceptar
     * @param evt evento
     */                                    
    @Override
    public void btnAceptarClic(ActionEvent evt) {
        JComboBox comboTipo = this.ventana.verComboTipo();
        JTextField txtNumero = this.ventana.verCampoNumero();
        String tipo = ((ModeloComboTipoTelefono)comboTipo.getModel()).obtenerTipoTelefono().toString();
        String numero = txtNumero.getText().trim();
        if (this.telefono == null) { //nuevo teléfono
            String resultado = this.persona.agregarTelefono(tipo, numero);
            JOptionPane.showMessageDialog(this.ventana, resultado, "Agenda", JOptionPane.INFORMATION_MESSAGE);
            if (resultado.equals(Persona.MENSAJE_ALTA_TELEFONO_OK))
                this.ventana.setVisible(false);
        }
        else { //modificación
            String resultado = this.persona.modificarTelefono(this.telefono, tipo, numero);
            JOptionPane.showMessageDialog(this.ventana, resultado, "Agenda", JOptionPane.INFORMATION_MESSAGE);
            if (resultado.equals(Persona.MENSAJE_MODIFICAR_TELEFONO_OK))
                this.ventana.setVisible(false);
        }
    }    
}
