/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package interfaces;

import java.time.LocalDate;
import java.util.List;
import modelos.Persona;

/**
 * Interfaz para los gestores de personas
 */
public interface IGestorPersonas {
    public static final String MENSAJE_ERROR = "Error al buscar personas";
    public static final String MENSAJE_PERSONA_CREADA = "Persona creada";
    public static final String MENSAJE_PERSONA_CREADA_ERROR = "El apellido y/o nombre están vacíos";
    public static final String MENSAJE_PERSONA_BORRADA = "Persona borrada";
    public static final String MENSAJE_PERSONA_BORRADA_ERROR = "No puede borrar la persona. Tiene teléfonos asociados";            
    
    /**
     * Devuelve una lista de personas cuyo apellido comience con la cadena que se especifica
     * Si la cadena es vacía, la lista contiene todas las personas creadas
     * Si no encuentra ninguna persona con esas características, devuelve una lista vacía
     * Si hay algún problema con la BD, devuelve null
     * @param cadena comienzo del apellido
     * @return List<Persona>  - lista con las personas cuyo apellido empiece con la cadena especificada
     * @see Persona
    */                                                        
    public List<Persona> buscar(String cadena);

    /**
     * Crea una Persona nueva en la base de datos
     * @param apellidos apellidos de la persona a dar de alta
     * @param nombres nombres de la persona a dar de alta
     * @param fechaNacimiento fecha de nacimiento de la persona a dar de alta
     * @param direccion dirección de la persona a dar de alta
     * @param correo correo de la persona a dar de alta
     * @return String  - cadena con el estado de la operación 
     * @see Persona
    */                                                            
    public String altaPersona(String apellidos, String nombres, LocalDate fechaNacimiento, String direccion, String correo);

    /**
     * Borra la Persona especificada de la base de datos
     * @param unaPersona persona a borrar
     * @return String  - cadena con el estado de la operación
     * @see Persona
    */                                                                
    public String borrarPersona(Persona unaPersona);

    /**
     * Devuelve una cadena con las personas que cumplen año hoy
     * @return String  - cadena con las personas que cumplen año hoy
     * @see Persona
    */                                                                    
    public String hoyCumple();

    /**
     * Devuelve una lista con el calendario de cumpleaños de todas las personas
     * La lista tiene la forma: dia/mes/nombre
     * Si no hay personas 
     * Si se produce algún error, devuelve null
     * @return List<String>  - lista con el calendario de cumpleaños de todas las personas
    */                                                                        
    public List<String> calendario();
}
