/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package modelos;

import java.sql.Connection;
import java.sql.DriverManager;

/**
 * Clase para realizar una conexión a un servidor SQL Server
*/       
public class ConexionSQLServer extends Conexion {
    private static ConexionSQLServer instancia;
    
    /**
     * Constructor
     * Este constructor asigna el valor a las variables url, driver, servidor, puerto, bd, usuario y clave (definidas en TipoConexion)
     * y luego carga la implementación del driver en memoria (a este proceso se lo llama "Registro del driver)
     * Después de cargar el driver en memoria, se establece una conexión a la BD mediante el método DriverManager.getConnection()
     * http://www.tutorialspoint.com/jdbc/jdbc-db-connections.htm
     * @param servidor servidor al cual conectarse
     * @param puerto puerto en el cual escucha el servidor
     * @param bd base de datos a la cual conectarse
     * @param usuario usuario con el cual conectarse
     * @param clave clave del usuario que hace la conexión
     * @throws Exception en caso de no poder realizar la conexión
    */        
    private ConexionSQLServer(String servidor, String puerto, String bd, String usuario, String clave) throws Exception {
        super("jdbc:sqlserver://", "com.microsoft.sqlserver.jdbc.SQLServerDriver");
        Connection cnn = DriverManager.getConnection(this.verURL() + servidor + ":" + puerto + ";DatabaseName=" + bd, usuario, clave);
        asignarConnection(cnn);            
    }
    
    /**
     * Método estático que permite crear una única instancia de ConexionSQLServer
     * @param servidor servidor al cual conectarse
     * @param puerto puerto en el cual escucha el servidor
     * @param bd base de datos a la cual conectarse
     * @param usuario usuario con el cual conectarse
     * @param clave clave del usuario que hace la conexión
     * @return ConexionSQLServer
     * @throws Exception en caso de no poder realizar la conexión
    */                                                                
    public static ConexionSQLServer crear(String servidor, String puerto, String bd, String usuario, String clave) throws Exception {
        if (instancia == null) 
            instancia = new ConexionSQLServer(servidor, puerto, bd, usuario, clave);
        return instancia;        
    }           
       
}
