/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package modelos;

import interfaces.IControladorLogin;
import interfaces.IGestorPersonas;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;

/**
 * Clase para mostrar en una tabla las distintas personas
 */
public class ModeloTablaPersonas extends AbstractTableModel {
    private List<String> nombresColumnas = new ArrayList<>();        
    //colección para guardar los nombres de las columnas

    private List<Persona> personas = new ArrayList<>();
        
    /**
     * Constructor
    */                                                        
    public ModeloTablaPersonas(String cadenaBusqueda) {
        this.nombresColumnas.add("IDPersona");
        this.nombresColumnas.add("Personas");  
        IGestorPersonas gp = GestorPersonas.crear();
        try {
            this.personas = gp.buscar(cadenaBusqueda);
        }
        catch(Exception e) {
            JOptionPane.showMessageDialog(null, GestorPersonas.MENSAJE_ERROR, IControladorLogin.TITULO_APLICACION, JOptionPane.ERROR_MESSAGE);        
        }
    }         

    /**
    * Obtiene el valor de la celda especificada
    * @param fila fila de la celda
    * @param columna columna de la celda
    * @return Object  - valor de la celda
    */                        
    @Override
    public Object getValueAt(int fila, int columna) {
        if (this.personas != null) {
            Persona unaPersona = this.personas.get(fila);
            if (columna == 0)
                return unaPersona.verID();
            else 
                return unaPersona.verApellidos() + ", " + unaPersona.verNombres();
        }
        else
            return null;
    }

    /**
    * Obtiene la cantidad de columnas de la tabla
    * @return int  - cantidad de columnas de la tabla
    */                            
    @Override
    public int getColumnCount() { 
        return this.nombresColumnas.size();
    }

    /**
    * Obtiene la cantidad de filas de la tabla
    * @return int  - cantidad de filas de la tabla
    */                        
    @Override
    public int getRowCount() { 
        return this.personas.size();
    }

    /**
    * Obtiene el nombre de una columna
    * @param columna columna sobre la que se quiere obtener el nombre
    * @return String  - nombre de la columna especificada
    */                        
    @Override
    public String getColumnName(int columna) {
        return this.nombresColumnas.get(columna);
    }

    /**
    * Obtiene el tipo de datos de una columna
    * @param columna columna sobre la que se quiere obtener el tipo de datos
    * @return Class  - clase de la columna especificada
    */                    
    @Override
    public Class getColumnClass(int columna) {
        if (columna == 0)
            return Integer.class;
        else
            return String.class;
    } 

    /**
    * Devuelve la Persona correspondiente a la posición especificada
    * @param indice lugar que ocupa la Persona a devolver
    * @return Persona  - objeto Persona correspondiente a la posición que se especifica
    * @see Persona
    */        
    public Persona obtenerPersona(int indice) {
        if (this.personas != null)
            return this.personas.get(indice);
        else
            return null;
    }       
}
