/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package modelos;

import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;

/**
 * Clase para mostrar en una tabla los teléfonos de una persona
 */
public class ModeloTablaTelefonos extends AbstractTableModel {
    private List<String> nombresColumnas = new ArrayList<>();        
    //colección para guardar los nombres de las columnas

    //los datos los saca de la persona
    private List<Telefono> telefonos;

    /**
     * Constructor
     * @param persona persona a la que se muestra sus teléfonos
    */                                                        
    public ModeloTablaTelefonos(Persona persona) {
        this.nombresColumnas.add("IDPersona");
        this.nombresColumnas.add("IDTelefono");
        this.nombresColumnas.add("Tipo");
        this.nombresColumnas.add("Número");   
        if (persona != null) 
            this.telefonos = persona.verTelefonos();
        else 
            this.telefonos = new ArrayList<>();
    }         

    /**
    * Obtiene el valor de la celda especificada
    * @param fila fila de la celda
    * @param columna columna de la celda
    * @return Object  - valor de la celda
    */                        
    @Override
    public Object getValueAt(int fila, int columna) {
        if (this.telefonos != null) {
            Telefono telefono = this.telefonos.get(fila);
            if (columna == 2)
                return telefono.verTipo();
            else 
                return telefono.verNumero();
        }
        else
            return null;
    }

    /**
    * Obtiene la cantidad de columnas de la tabla
    * @return int  - cantidad de columnas de la tabla
    */                            
    @Override
    public int getColumnCount() { 
        return this.nombresColumnas.size();
    }

    /**
    * Obtiene la cantidad de filas de la tabla
    * @return int  - cantidad de filas de la tabla
    */                        
    @Override
    public int getRowCount() { 
        return this.telefonos.size();
    }

    /**
    * Obtiene el nombre de una columna
    * @param columna columna sobre la que se quiere obtener el nombre
    * @return String  - nombre de la columna especificada
    */                        
    @Override
    public String getColumnName(int columna) {
        return this.nombresColumnas.get(columna);
    }

    /**
    * Obtiene el tipo de datos de una columna
    * @param columna columna sobre la que se quiere obtener el tipo de datos
    * @return Class  - clase de la columna especificada
    */                    
    @Override
    public Class getColumnClass(int columna) {
        return String.class;
    } 

    /**
    * Devuelve el teléfono correspondiente a la posición especificada
    * @param indice lugar que ocupa el teléfono a devolver
    * @return Telefono  - objeto Telefono correspondiente a la posición que se especifica
    */        
    public Telefono obtenerTelefono(int indice) {
        if (this.telefonos != null)
            return this.telefonos.get(indice);
        else
            return null;
    }   
}
