package prueba;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class Persona {
    private Integer idPersona;
    private String apellidos;
    private String nombres;
    private LocalDate fechaNacimiento;
    private String direccion;
    private String correo;
    private List<Telefono> telefonos = new ArrayList<>();
        

    /** Constructor 
     * @param idPersona identificador de la persona
     * @param apellidos apellidos de la persona
     * @param nombres nombres de la persona
     * @param fechaNacimiento fecha de nacimiento de la persona
     * @param direccion dirección de la persona
     * @param email email de la persona
    */    
    public Persona (Integer idPersona, String apellidos, String nombres, LocalDate fechaNacimiento, String direccion, String email) {
        this.idPersona = idPersona;
        this.apellidos = apellidos;
        this.nombres = nombres;
        this.fechaNacimiento = fechaNacimiento;
        this.direccion = direccion;
        this.correo = email;    
    }

    /** Devuelve el identificador de la persona
     * @return int con el identificador de la persona
    */        
    public int verID() {
        return this.idPersona;
    }

    /** Asigna el identificador a la persona
     * @param idPersona identificador de la persona
    */            
    public void asignarID(int idPersona) {
        this.idPersona = idPersona;
    }

    /** Devuelve los apellidos de la persona
     * @return String con los apellidos de la persona
    */              
    public String verApellidos() {
        return this.apellidos;
    }

    /** Asigna los apellidos a la persona
     * @param apellidos apellidos de la persona
    */                
    public void asignarApellidos(String apellidos) {
        this.apellidos = apellidos;
    }

    /** Devuelve los nombres de la persona
     * @return String con los nombres de la persona
    */            
    public String verNombres() {
        return this.nombres;
    }

    /** Asigna los nombres a la persona
     * @param nombres nombres de la persona
    */                
    public void asignarNombres(String nombres) {
        this.nombres = nombres;
    }

    /** Devuelve la fecha de nacimiento de la persona
     * @return LocalDate con la fecha de nacimiento de la persona
    */            
    public LocalDate verFecNac() {
        return this.fechaNacimiento;
    }

    /** Asigna la fecha de nacimiento a la persona
     * @param fechaNacimiento fecha de nacimiento de la persona
    */                
    public void asignarFecNac(LocalDate fechaNacimiento) {
        this.fechaNacimiento = fechaNacimiento;
    }

    /** Devuelve la dirección de la persona
     * @return String con la dirección de la persona
    */            
    public String verDireccion() {
        return this.direccion;
    }

    /** Asigna la dirección a la persona
     * @param direccion dirección de la persona
    */                
    public void asignarDireccion(String direccion) {
        this.direccion = direccion;
    }

    /** Devuelve el correo de la persona
     * @return String con el correo de la persona
    */            
    public String verCorreo() {
        return this.correo;
    }

    /** Asigna el correo a la persona
     * @param correo correo de la persona
    */                
    public void asignarCorreo(String correo) {
        this.correo = correo;
    }
    
    public void mostrar() {
        System.out.println(this.apellidos + ", " + this.nombres);
        if (this.fechaNacimiento != null) {
            String patron = "dd/MM/yyyy";   
            String fFormateada = this.fechaNacimiento.format(DateTimeFormatter.ofPattern(patron));
            System.out.println("Fecha de nacimiento: " + fFormateada);
        }
        else
            System.out.println("Fecha de nacimiento: " + "-");
        System.out.println("Dirección: " + (this.direccion == null ? this.direccion : "-"));
        System.out.println("Correo: " + (this.correo == null ? this.correo : "-"));
        System.out.println("Teléfonos:");
        for(Telefono t : this.telefonos) {
            t.mostrar();
            System.out.println();
        }
    }

    /**
     * Obtiene el hashcode del objeto teniendo en cuenta idPersona
     * @return int  - hashcode del objeto
     */
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 97 * hash + Objects.hashCode(this.idPersona);
        return hash;
    }

    /**
     * Determina si 2 personas son iguales según idPersona
     * @param obj objeto Persona
     * @return boolean  - resultado de comparar si 2 personas son iguales
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Persona other = (Persona) obj;
        if (!Objects.equals(this.idPersona, other.idPersona)) {
            return false;
        }
        return true;
    }
                    
    /**
     * Agrega un teléfono a la persona
     * @param unTelefono teléfono a agregar
     * @see Telefono
    */                            
    public void agregarTelefono (Telefono unTelefono) {
        if (!this.telefonos.contains(unTelefono))
            this.telefonos.add(unTelefono);
    }

}
