/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package prueba;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JOptionPane;

public class Principal4 {
    private static List<Persona> personas = new ArrayList<>();
    
    public static void main(String[] args) {                
        String usuario = "Estudiante";
        String clave = "estudiante";
        String servidor = "localhost";
        String puerto = "3306";
        String url = "jdbc:mysql://";
        String driver = "com.mysql.jdbc.Driver";        
        String bd = "Agenda";        
        
        try {
            Connection cnn = conectar(url, driver, servidor, puerto, bd, usuario, clave);
            
            try {
                ResultSet rs = consultarPersonas_Statement(cnn);                
//                ResultSet rs = consultarPersonas_PreparedStatement(cnn);
//                ResultSet rs = consultarPersonas_CallableStatement(cnn);
                procesarResultSet(rs);
                mostrarPersonas();                
            }
            catch(SQLException e) {
                JOptionPane.showMessageDialog(null, "Error al leer las personas", "Agenda", JOptionPane.ERROR_MESSAGE);                    
            }                        
        }
        catch(ClassNotFoundException e) {
            JOptionPane.showMessageDialog(null, "No se encontró la clase del driver", "Agenda", JOptionPane.ERROR_MESSAGE);
        }
        catch(SQLException e) {
            String mensajeError = "Error de conexión\n" + "Servidor: " + servidor + "\nPuerto: " + puerto + "\nBD: " + bd + "\nUsuario: " + usuario;
            JOptionPane.showMessageDialog(null, mensajeError, "Agenda", JOptionPane.ERROR_MESSAGE);                    
        }        
    }    

    /**
     * Realiza la conexión a la BD
     * @param url url a la BD
     * @param driver driver para el motor
     * @param servidor servidor de BD
     * @param puerto puerto del servicio
     * @param bd nombre de la BD
     * @param usuario usuario
     * @param clave clave
     * @return Connection  - objeto Connection
     * @throws ClassNotFoundException 
     * @throws SQLException 
     */
    private static Connection conectar(String url, String driver, String servidor, String puerto, String bd, String usuario, String clave) throws ClassNotFoundException, SQLException {
        Class.forName(driver);
        //registro de un driver JDBC para MySQL
        
        return DriverManager.getConnection(url + servidor + ":" + puerto + "/" + bd + "?noAccessToProcedureBodies=true", usuario, clave);        
    }
    
    /**
     * Consulta las personas empleando un objeto Statement
     * @param cnn objeto Connection con la conexión establecida
     * @return ResultSet  - resultset con el resultado de la consulta
     * @throws SQLException 
     */
    private static ResultSet consultarPersonas_Statement(Connection cnn) throws SQLException {        
        Statement stmt = cnn.createStatement();
        
        String cadena = "select Personas.IDPersona, Apellidos, Nombres, FechaNacimiento, Domicilio, Email, IDTelefono, Tipo, Numero ";
        cadena += "from Personas join Telefonos on Personas.IDPersona = Telefonos.IDPersona ";
        cadena += "order by Personas.IDPersona";
        
        ResultSet rs = stmt.executeQuery(cadena);

        return rs;                   
    }
    
    /**
     * Consulta las personas empleando un objeto PreparedStatement
     * @param cnn 
     */
    private static ResultSet consultarPersonas_PreparedStatement(Connection cnn) throws SQLException {        
        String cadena = "select Personas.IDPersona, Apellidos, Nombres, FechaNacimiento, Domicilio, Email, IDTelefono, Tipo, Numero ";
        cadena += "from Personas join Telefonos on Personas.IDPersona = Telefonos.IDPersona ";
        cadena += "order by Personas.IDPersona";
        
        PreparedStatement prepstmt = cnn.prepareStatement(cadena);
        
        ResultSet rs = prepstmt.executeQuery();
            
        return rs;        
    }
    
    /**
     * Consulta las personas empleando un objeto CallableStatement
     * @param cnn 
     */
    private static ResultSet consultarPersonas_CallableStatement(Connection cnn) throws SQLException {        
        String cadena = "select Personas.IDPersona, Apellidos, Nombres, FechaNacimiento, Domicilio, Email, IDTelefono, Tipo, Numero ";
        cadena += "from Personas join Telefonos on Personas.IDPersona = Telefonos.IDPersona ";
        cadena += "order by Personas.IDPersona";
        
        CallableStatement calstmt = cnn.prepareCall(cadena);
        
        ResultSet rs = calstmt.executeQuery();
        
        return rs;
    }
    
    /**
     * Procesa el ResultSet
     * @param rs ResultSet con los resultados de la consulta
     * @throws SQLException 
     */
    private static void procesarResultSet(ResultSet rs) throws SQLException {
        while (rs.next()) {
            Integer idPersona = rs.getInt("IDPersona");
            String apellidos = rs.getString("Apellidos");
            String nombres = rs.getString("Nombres");
            LocalDate fechaNacimiento = null;
            if (rs.getDate("FechaNacimiento") != null)
                fechaNacimiento = Instant.ofEpochMilli(rs.getDate("FechaNacimiento").getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
            String domicilio = rs.getString("Domicilio");
            String email = rs.getString("Email");
            int idTelefono = rs.getInt("IDTelefono");
            String tipo = rs.getString("Tipo");
            String numero = rs.getString("Numero");
            Persona unaPersona = new Persona(idPersona, apellidos, nombres, fechaNacimiento, domicilio, email);
            if (!personas.contains(unaPersona))
                personas.add(unaPersona);                   
            agregarTelefono(idPersona, idTelefono, tipo, numero);
        }
        rs.close();        
    }
    
    /**
     * Muestra las personas
     */
    public static void mostrarPersonas() {                        
        for(Persona p : personas) {
            p.mostrar();
            System.out.println();
        }
    }
    
    /**
     * Agrega el teléfono especificado a la persona especificada
     * @param idPersona id de la persona
     * @param idTelefono id del teléfono
     * @param tipo tipo de teléfono
     * @param numero número de teléfono
     */
    private static void agregarTelefono(Integer idPersona, int idTelefono, String tipo, String numero) {
        Persona unaPersona = buscarPersona(idPersona);
        if (unaPersona != null) {
            TipoTelefono t = obtenerTipo(tipo);
            Telefono unTelefono = new Telefono(idTelefono, idPersona, t, numero);
            unaPersona.agregarTelefono(unTelefono);
        }
    }
    
    /**
     * Devuelve la persona con el id especificado. Si no hay ninguna con ese id devuelve null
     * @param idPersona idPersona a buscar
     * @return Persona  - persona con el id especificado
     */
    private static Persona buscarPersona(Integer idPersona) {
        for(Persona p : personas) {
            if (p.verID() == idPersona)
                return p;
        }
        return null;
    }
    
    /**
     * Transforma una cadena en el valor de su enumeración correspondiente
     * @param tipo
     * @return TipoTelefono
     */
    private static TipoTelefono obtenerTipo(String tipo) {
        if (tipo.trim().equals("Particular"))
            return TipoTelefono.PARTICULAR;
        else if (tipo.trim().equals("Celular"))
            return TipoTelefono.CELULAR;
        else if (tipo.trim().equals("Trabajo"))
            return TipoTelefono.TRABAJO;
        else
            return TipoTelefono.OTRO;
    }
}
